﻿using System;
using System.Windows.Forms;
using Example_Web_Service_Client.technicianReference;
using System.Text;
using System.IO;
using System.Net;

namespace Example_Web_Service_Client
{
    public partial class MainForm : Form
    {
        private static string serviceLocation = "http://contentservices.installernet.com/vtt/2.0/Unencrypted/VTTechnician20_UnEn.dll/SOAP";
        private static string fitGuideApplicationId = "{79483C46-2637-45F1-9209-573DDF985BFC}";

        private vtVehicleMake[] makes;
        private vtVehicleModel[] models;
        private int[] years;
        private vtVehicleBodyType[] bodyTypes;
        private vtVehicleTrim2[] trimLevels;
        private vtRetailerVehicleResponse9 vehicleResponse;

        private string sessionId;
        private int fitGuideId;

        private bool updatesLocked;

        public MainForm()
        {
            InitializeComponent();
        }

        /* UI event handlers */

        private void button1_Click(object sender, EventArgs e)
        {
            Cursor saveCursor = Cursor.Current;
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                Login();
            }
            finally
            {
                Cursor.Current = saveCursor;
            }
        }

        private void makeCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!updatesLocked)
                PopulateVehicleModels();
        }

        private void modelCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!updatesLocked)
                PopulateVehicleYears();
        }

        private void yearCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!updatesLocked)
                PopulateVehicleBodyTypes();
        }

        private void bodyTypeCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!updatesLocked)
                PopulateVehicleTrimLevels();
        }

        private void lookUpButton_Click(object sender, EventArgs e)
        {
            Cursor saveCursor = Cursor.Current;
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                LookUpVehicle();
            }
            finally
            {
                Cursor.Current = saveCursor;
            }
        }

        private void categoryTree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (categoryTree.SelectedNode != null)
                PopulateProductListView();
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            LogOff();
        }

        /* web service method calls */

        private void Login()
        {
            using (vtUserServicesBinding userServices = new vtUserServicesBinding { Url = serviceLocation })
            {
                vtLoginInfo4 request = new vtLoginInfo4
                {
                    Username = usernameText.Text,
                    Password = passwordText.Text,
                    ApplicationID = fitGuideApplicationId
                };
                vtLoginResult17 result;
                if (userServices.Login23(request, out result))
                {
                    sessionId = result.SessionId;
                    fitGuideId = int.Parse(fitGuideIdEdit.Text);
                    PopulateVehicleMakes();
                }
            }
        }

        private void LogOff()
        {
            if (!string.IsNullOrEmpty(sessionId))
                using (vtUserServicesBinding userServices = new vtUserServicesBinding { Url = serviceLocation })
                    userServices.Logoff(sessionId);
        }

        private void PopulateVehicleMakes()
        {
            updatesLocked = true;
            try
            {
                using (vtVehicleServicesBinding vehicleServices = new vtVehicleServicesBinding { Url = serviceLocation })
                {
                    makes = vehicleServices.GetRetailerVehicleMakesDown(sessionId, fitGuideId, out models, out years, out bodyTypes, out trimLevels);
                    PopulateMakeCombo(makes);
                    PopulateModelCombo(models);
                    PopulateYearCombo(years);
                    PopulateBodyTypeCombo(bodyTypes);
                    PopulateTrimLevelCombo(trimLevels);
                }
            }
            finally
            {
                updatesLocked = false;
            }
        }

        private void PopulateVehicleModels()
        {
            updatesLocked = true;
            try
            {
                using (vtVehicleServicesBinding vehicleServices = new vtVehicleServicesBinding { Url = serviceLocation })
                {
                    models = vehicleServices.GetRetailerVehicleModelsDown(sessionId, fitGuideId, SelectedMakeId(),
                        out years, out bodyTypes, out trimLevels);
                    PopulateModelCombo(models);
                    PopulateYearCombo(years);
                    PopulateBodyTypeCombo(bodyTypes);
                    PopulateTrimLevelCombo(trimLevels);
                }
            }
            finally
            {
                updatesLocked = false;
            }
        }

        private void PopulateVehicleYears()
        {
            updatesLocked = true;
            try
            {
                using (vtVehicleServicesBinding vehicleServices = new vtVehicleServicesBinding { Url = serviceLocation })
                {
                    years = vehicleServices.GetRetailerVehicleYearsDown(sessionId, fitGuideId, SelectedMakeId(),
                        SelectedModelId(), out bodyTypes, out trimLevels);
                    PopulateYearCombo(years);
                    PopulateBodyTypeCombo(bodyTypes);
                    PopulateTrimLevelCombo(trimLevels);
                }
            }
            finally
            {
                updatesLocked = false;
            }
        }

        private void PopulateVehicleBodyTypes()
        {
            updatesLocked = true;
            try
            {
                using (vtVehicleServicesBinding vehicleServices = new vtVehicleServicesBinding { Url = serviceLocation })
                {
                    bodyTypes = vehicleServices.GetRetailerVehicleBodyTypesDown(sessionId, fitGuideId, SelectedMakeId(),
                        SelectedModelId(), SelectedVehicleYear(), out trimLevels);
                    PopulateBodyTypeCombo(bodyTypes);
                    PopulateTrimLevelCombo(trimLevels);
                }
            }
            finally
            {
                updatesLocked = false;
            }
        }

        private void PopulateVehicleTrimLevels()
        {
            updatesLocked = true;
            try
            {
                using (vtVehicleServicesBinding vehicleServices = new vtVehicleServicesBinding { Url = serviceLocation })
                {
                    trimLevels = vehicleServices.GetRetailerVehicleTrims(sessionId, fitGuideId, SelectedMakeId(),
                        SelectedModelId(), SelectedVehicleYear(), SelectedBodyTypeId());
                    PopulateTrimLevelCombo(trimLevels);
                }
            }
            finally
            {
                updatesLocked = false;
            }
        }

        private void LookUpVehicle()
        {
            using (vtProductServicesBinding productServices = new vtProductServicesBinding { Url = serviceLocation })
            {
                vtRetailerVehicleRequest4 request = new vtRetailerVehicleRequest4
                {
                    VehicleId = SelectedVehicleId(),
                    RetailerId = fitGuideId,
                    ProductCategories = true,
                    Products = true
                };
                vehicleResponse = productServices.GetRetailerVehicleInformation11(sessionId, request);
                PopulateVehicleControls();
            }

        }

        private void LookUpAttributes()
        {
            using (vtVehicleServicesBinding vehicleServices = new vtVehicleServicesBinding { Url = serviceLocation })
            {
                //Get Values
                vtAttributeValue[] attribArray = new vtAttributeValue[2];
                attribArray[0] = new vtAttributeValue();
                attribArray[0].AttributeID = 12905; // OBDII connector location
                attribArray[1] = new vtAttributeValue();
                attribArray[1].AttributeID = 12906; // OBDII connector picture
                vehicleServices.GetAttributeValues(sessionId, SelectedVehicleId(), ref attribArray);
                
                //Populate list view
                attributesListView.BeginUpdate();
                try
                {
                    attributesListView.Items.Clear();
                    foreach (vtAttributeValue av in attribArray)
                    {
                        ListViewItem item = new ListViewItem { Text = Convert.ToString(av.AttributeID) };
                        item.SubItems.Add(av.Value);
                        attributesListView.Items.Add(item);
                    }
                }
                finally
                {
                    attributesListView.EndUpdate();
                }

            }
        }

        /* populating UI elements from web service data */

        private void PopulateMakeCombo(vtVehicleMake[] makes)
        {
            makeCombo.BeginUpdate();
            try
            {
                makeCombo.Items.Clear();
                foreach (vtVehicleMake make in makes)
                    makeCombo.Items.Add(make.Make);
                makeCombo.SelectedIndex = 0;
            }
            finally
            {
                makeCombo.EndUpdate();
            }
        }

        private void PopulateModelCombo(vtVehicleModel[] models)
        {
            modelCombo.BeginUpdate();
            try
            {
                modelCombo.Items.Clear();
                foreach (vtVehicleModel model in models)
                    modelCombo.Items.Add(model.Model);
                modelCombo.SelectedIndex = 0;
            }
            finally
            {
                modelCombo.EndUpdate();
            }
        }

        private void PopulateYearCombo(int[] years)
        {
            yearCombo.BeginUpdate();
            try
            {
                yearCombo.Items.Clear();
                foreach (int year in years)
                    yearCombo.Items.Add(year.ToString());
                yearCombo.SelectedIndex = 0;
            }
            finally
            {
                yearCombo.EndUpdate();
            }
        }

        private void PopulateBodyTypeCombo(vtVehicleBodyType[] bodyTypes)
        {
            bodyTypeCombo.BeginUpdate();
            try
            {
                bodyTypeCombo.Items.Clear();
                foreach (vtVehicleBodyType bodyType in bodyTypes)
                    bodyTypeCombo.Items.Add(bodyType.BodyType);
                bodyTypeCombo.SelectedIndex = 0;
            }
            finally
            {
                bodyTypeCombo.EndUpdate();
            }
        }

        private void PopulateTrimLevelCombo(vtVehicleTrim2[] trimLevels)
        {
            trimLevelCombo.BeginUpdate();
            try
            {
                trimLevelCombo.Items.Clear();
                foreach (vtVehicleTrim2 trimLevel in trimLevels)
                    trimLevelCombo.Items.Add(trimLevel.TrimLevel);
                trimLevelCombo.SelectedIndex = 0;
            }
            finally
            {
                trimLevelCombo.EndUpdate();
            }
        }

        private void PopulateFitControls()
        {
            deckSizeLabel.Text = vehicleResponse.RadioInformation.Size;
            StringBuilder sb = new StringBuilder();
            foreach (vtOpeningInformation item in vehicleResponse.SpeakerInformation)
                sb.AppendLine(string.Format("{0} - Size: {1} Depth: {2}", item.OpeningName, item.Size, item.Depth));
            speakerSpecsText.Text = sb.ToString();
        }

        private void PopulateVehicleImage()
        {
            if (vehicleResponse.ImageInformation.Length > 0)
            {
                string fileName = Path.ChangeExtension(vehicleResponse.ImageInformation[0].Value, ".jpg");
                string url = String.Format("http://contentdocs.installernet.com/carpics/{0}", fileName);
                string filePath = Path.GetTempPath() + fileName;
                File.Delete(filePath);
                using (WebClient client = new WebClient())
                    client.DownloadFile(url, filePath);
                pictureBox1.Image = System.Drawing.Bitmap.FromFile(filePath);
            }
        }

        private void PopulateVehicleControls()
        {
            PopulateCategoryTree(vehicleResponse.ProductCategories, null);
            PopulateFitControls();
            PopulateVehicleImage();
        }

        private bool CategoryHasProducts(vtProductCategory2 productCategory)
        {
            foreach (vtProduct11 product in vehicleResponse.Products)
                if (product.CategoryID == productCategory.CategoryId)
                    return true;

            foreach (vtProductCategory2 category in productCategory.Categories)
                if (CategoryHasProducts(category))
                    return true;

            return false;
        }

        private void PopulateCategoryTree(vtProductCategory2[] productCategories, TreeNode parentNode)
        {
            categoryTree.BeginUpdate();
            try
            {
                if (parentNode == null)
                    categoryTree.Nodes.Clear();
                foreach (vtProductCategory2 productCategory in productCategories)
                    if (CategoryHasProducts(productCategory))
                    {
                        TreeNode node;
                        if (parentNode == null)
                            node = categoryTree.Nodes.Add(productCategory.Category);
                        else
                            node = parentNode.Nodes.Add(productCategory.Category);
                        node.Tag = productCategory.CategoryId;
                        PopulateCategoryTree(productCategory.Categories, node);
                    }
            }
            finally
            {
                if (parentNode == null)
                    categoryTree.ExpandAll();
                categoryTree.EndUpdate();
            }
        }

        private void PopulateProductListView()
        {
            productListView.BeginUpdate();
            try
            {
                productListView.Items.Clear();
                foreach (vtProduct11 product in vehicleResponse.Products)
                    if (product.CategoryID == SelectedCategoryId())
                    {
                        ListViewItem item = new ListViewItem { Text = product.PartNumber };
                        item.SubItems.Add(product.Description);
                        productListView.Items.Add(item);
                    }
            }
            finally
            {
                productListView.EndUpdate();
            }
        }

        /* helper methods to make the example code a little easier to read */

        private int SelectedMakeId()
        {
            return makes[makeCombo.SelectedIndex].MakeID;
        }

        private int SelectedModelId()
        {
            return models[modelCombo.SelectedIndex].ModelID;
        }

        private int SelectedVehicleYear()
        {
            return years[yearCombo.SelectedIndex];
        }

        private int SelectedBodyTypeId()
        {
            return bodyTypes[bodyTypeCombo.SelectedIndex].BodyTypeID;
        }

        private int SelectedVehicleId()
        {
            return trimLevels[trimLevelCombo.SelectedIndex].VehicleId;
        }

        private int SelectedCategoryId()
        {
            return (int)categoryTree.SelectedNode.Tag;
        }

        private void attributesButton_Click(object sender, EventArgs e)
        {
            Cursor saveCursor = Cursor.Current;
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                LookUpAttributes();
            }
            finally
            {
                Cursor.Current = saveCursor;
            }
        }
    }
}
